""".. include:: ../../README.md"""  # noqa: D415

from cmi_docx.document import ExtendDocument  # noqa: F401
from cmi_docx.paragraph import ExtendParagraph, FindParagraph  # noqa: F401
from cmi_docx.run import ExtendRun, FindRun  # noqa: F401
from cmi_docx.styles import ParagraphStyle, RunStyle, TableStyle  # noqa: F401
from cmi_docx.table import ExtendCell  # noqa: F401
